// Slide Simple WebDAV client example
import java.io.*;
import java.net.MalformedURLException;
import java.lang.*;

import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpURL;
import org.apache.commons.httpclient.HttpsURL;
import org.apache.webdav.lib.WebdavResource;
import org.apache.webdav.lib.WebdavResources;

import java.util.Enumeration;
import javax.swing.text.Element;
import java.util.ArrayList;

/**
 * This class tests the Slide WebDAV client library found at jakarta.apache.org/slide
 *
 * It connects to a hard-coded location at the DFS filesystem at CERN, and performs
 * disk operations like creating file, showing directories and files, saving files 
 * to the local filesystem ++
 *
 * @author Ola Holmestad Ola.Holmestad@cern.ch
 * @version %I% %G%
 *
 */

public class SlideTest implements Runnable {

	Thread t;
	WebdavResource wdr = null;
	boolean run = true;
	String password = null;
	BufferedReader br = null;


	/**
	 * This constructor creates a new thread to handle keyboard input. It then tries to 
	 * connect to the WebDAVResource hardcoded in an HttpsURL. If a successfull 
	 * connection is created, the thread is started and the user can issue keyboard inputs
	 * 
	 * @param password	the password for the hardcoded resource
	 *
	 */

    	public SlideTest(String password) {

		t = new Thread(this);	
		this.password = password;

		//for the keyboard input
		br = new BufferedReader(new InputStreamReader(System.in));

		try {
	
		    //creating an HttpsURL to the DFS file system at CERN
	            HttpsURL hrl = new HttpsURL("https://dfs.cern.ch/dfs/Users/h/holmesta/");
        	    hrl.setUserinfo("holmesta", password);
	            wdr = new WebdavResource(hrl);

		    //if not successfull connection, exit, else print confirmation message and start thread

        	    if (wdr == null) {
                	System.out.println("No WebdavResource at "+hrl);
	                System.exit(0);
        	    }

	            else {

        	        System.out.println("Successfull connection to webdav resource: "+hrl.getURI());
	
			t.start();

	 	    }

		}

		catch(MalformedURLException mue) {

        	        System.out.println("MalformedURLException thrown");
                	mue.printStackTrace();
        
		}

	        catch(HttpException he) {

        	        System.out.println("HttpException thrown");
                	he.printStackTrace();

	        }

        	catch(IOException ioe) {

                	System.out.println("IOException thrown");
	                ioe.printStackTrace();

        	}

    	}

	/**
	 * Main method only creating an instance of the class
	 *
 	 * @param args	input needs to be the password for the DFS access
	 *
	 */

    	public static void main (String[] args) {

		SlideTest st = new SlideTest(args[0]);

    	}    

	/**
	 * Basically a method that calls the receiveInput()-method as long as the user hasn't
	 * issued the 'quit' command
	 *
	 */

    	public void run() {

		int i=0;
		run = true;
		
		//run will be true as long as the user does not enter 'quit' at the command line

		while (run) {

			receiveInput();

		}

    	}

	/**
  	 * This method waits for the input from the command line and then calls the actions()-method
 	 *
	 */

	public void receiveInput() {

		try {

			System.out.println();
			System.out.print("Enter input: ");
			String action = br.readLine();

			actions(action);
				
		}

		catch (IOException ioe) {

			ioe.printStackTrace();

		}

    	}

	/**
	 * Calls the right method depending on the user input. Either list, creatfile, todir,
	 * quit, createdir, save or import (currently not working).
	 *
	 * If the action is not listed here, the user is prompted for a new input
	 *
	 * @param action the keyboard input string from the user
	 */

	public void actions(String action) {

		if (action.equals("list")) 
	
			actionList();

		else if (action.equals("createfile"))

			actionCreateFile();	

		else if (action.equals("todir")) 

			actionToDirectory();

		else if (action.equals("quit"))

			actionQuit();

		else if (action.equals("createdir"))

			actionCreateDir();

		else if (action.equals("save")) 

			actionSave();

		else if (action.equals("import")) 

			actionImport();

    	}

	/**
	 * Saves a file to the local file system
	 *
	 * Demands the user for the path to the file that should be saved and saves this file 
	 * at a local location demanded by the user. The user has to know the structure of the 
 	 * local file system to be able to give a correct path with file name for the local
	 * file system
	 *
	 */

	public void actionSave() {

		System.out.println("Save file to local disk");

		try {

			System.out.print("Path to file that should be saved: ");
			String remotePath = br.readLine();

			System.out.print("Path to location with file name at local filesystem: ");
			String localPath = br.readLine();

			boolean success = wdr.getMethod(remotePath, new java.io.File(localPath));

			if (success) 
			
				System.out.println("File successfully saved at location "+localPath);

			else

				System.out.println("File not saved for some reason. Check that both paths are valid");

		}

		catch (IOException ioe) {

			ioe.printStackTrace();

		}


	}

	/** 
	 * Imports a file from the local file system. Does not currently work for some reason
	 *
	 */

	public void actionImport() {

		System.out.println("Import file from local disk");
		
		try {

			System.out.print("Give path to the local file: ");
			String localPath = br.readLine();

			//trying to create a FileInputStream from the local file and then use the putMethod()-method,
			//but this does not work for some strange reason

			FileInputStream fStream = new FileInputStream(localPath);
			System.out.println(wdr.getPath());
	
			if (wdr.putMethod(wdr.getPath(), fStream)) 
	
				System.out.println("First solution worked");
	
			else 
	
				System.out.println("Did not work either");


			//System.out.println(wdr.getPath());

		}

		catch (IOException ioe) {

			ioe.printStackTrace();

		}

	}

	/**
	 * Creates a new directory in the working directory for the WebdavResource
	 *
	 */

	public void actionCreateDir() {

		try {

			System.out.print("Directory name: ");
			String dirName = br.readLine();

			System.out.println(wdr.getPath()+dirName);

			if (wdr.mkcolMethod(wdr.getPath()+dirName)) 
				 System.out.println("Directory "+wdr.getPath()+dirName+" successfully created");
                	else
                        	System.out.println("No directory created");

		}

		catch (HttpException httpe) {

        	        httpe.printStackTrace();

	        }

        	catch (IOException ioe) {

                	ioe.printStackTrace();

	        }

        	receiveInput();

    	}

	/**
	 * Quits the command line interface
	 *
	 */

	public void actionQuit() {

		run = false;

	    }

	/**
	 * Creates a file with the specified file name from the first input and with the content specified
	 * in the second input
	 *
	 */

	public void actionCreateFile() {

		try {

			System.out.print("File name: ");
			String fileName = br.readLine();

			System.out.print("Contents: ");
			String contents = br.readLine();            

			if (wdr.putMethod(wdr.getPath()+"/"+fileName, contents))
        			System.out.println("File "+fileName+" successfully created");
		        else
        	        System.out.println("No file created");

		}

		catch (HttpException httpe) {

			httpe.printStackTrace();

		}

		catch (IOException ioe) {

			ioe.printStackTrace();

		}

        	receiveInput();    		

	}

	/**
	 * Sets the WebdavResource pointer to the new directory given by the user input
	 * 
	 * If a wrong directory is specified the system will exit, this should be changed to just give
	 * an error message
	 *
	 */

   	public void actionToDirectory() {

		System.out.print("Enter directory: ");

		try {
		
			String dir = br.readLine();

			HttpsURL hrl = new HttpsURL("https://dfs.cern.ch"+dir);
		        hrl.setUserinfo("holmesta", password);
        	        wdr = new WebdavResource(hrl);

                	if (wdr == null) {
	                	System.out.println("No WebdavResource at "+hrl);
		                System.exit(0);
        		}

	            	else {
	
        	        	System.out.println("Successfull connection to webdav resource: "+hrl.getURI());

			}

		}

		catch (HttpException httpe) {

			httpe.printStackTrace();

		}

		catch (IOException ioe) {

			ioe.printStackTrace();

		}

		receiveInput();

    	}

	/**
	 * Lists all the directories and all the files for this WebdavResource
	 *
	 */

	public void actionList() {

		//this method should list all directories and files in the current directory

		ArrayList<WebdavResource> directories = new ArrayList<WebdavResource>();
		ArrayList<WebdavResource> files = new ArrayList<WebdavResource>();

		try {
		
			WebdavResource[] resources = wdr.listWebdavResources();

			//loops through all the resources found by the listWebdavResources() call
			//and places them in the appriopriate ArrayList

			for (int i = 0; i < resources.length; i++) {

				if (resources[i].isCollection())

					directories.add(resources[i]);

				else 

					files.add(resources[i]);

			}

			//prints the information for the directories

			System.out.println();
			System.out.println("Directories:");
			System.out.println();

			for (int i = 0; i < directories.size(); i++) {

				System.out.println(((WebdavResource) directories.get(i)).getPath());

			}

			//prints the information for the files

			System.out.println();
			System.out.println("Files:");
			System.out.println();

			for (int i = 0; i < files.size(); i++) {

                	        System.out.println(((WebdavResource) files.get(i)).getPath());

	                }

		}

		catch (HttpException httpe) {

			httpe.printStackTrace();

		}

		catch (IOException ioe) {

			ioe.printStackTrace();

		}

		receiveInput();

    }

}
